@@ -14,8 +14,6 @@
  * limitations under the License.
  */
 
-#define CORE_PRIVATE
-
 #include "httpd.h"
 #include "http_log.h"
 #include "mpm_winnt.h"
@@ -24,7 +22,7 @@
 #include "apr_portable.h"
 #include "ap_regkey.h"
 
-static char  *display_name  = NULL;
+static const char *display_name  = NULL;
 static HANDLE stderr_thread = NULL;
 static HANDLE stderr_ready;
 
@@ -41,6 +39,7 @@ static DWORD WINAPI service_stderr_thread(LPVOID hPipe)
     apr_pool_t *p;
 
     apr_pool_create_ex(&p, NULL, NULL, NULL);
+    apr_pool_tag(p, "service_stderr_thread");
 
     errarg[0] = "The Apache service named";
     errarg[1] = display_name;
@@ -101,7 +100,7 @@ static DWORD WINAPI service_stderr_thread(LPVOID hPipe)
 
     if ((errres = GetLastError()) != ERROR_BROKEN_PIPE) {
         apr_snprintf(errbuf, sizeof(errbuf),
-                     "Win32 error %d reading stderr pipe stream\r\n",
+                     "Win32 error %lu reading stderr pipe stream\r\n",
                      GetLastError());
 
         ReportEvent(hEventSource, EVENTLOG_ERROR_TYPE, 0,
@@ -131,13 +130,11 @@ void mpm_nt_eventlog_stderr_flush(void)
 }
 
 
-void mpm_nt_eventlog_stderr_open(char *argv0, apr_pool_t *p)
+void mpm_nt_eventlog_stderr_open(const char *argv0, apr_pool_t *p)
 {
     SECURITY_ATTRIBUTES sa;
-    HANDLE hProc = GetCurrentProcess();
     HANDLE hPipeRead = NULL;
     HANDLE hPipeWrite = NULL;
-    HANDLE hDup = NULL;
     DWORD  threadid;
     apr_file_t *eventlog_file;
     apr_file_t *stderr_file;
@@ -155,13 +152,13 @@ void mpm_nt_eventlog_stderr_open(char *argv0, apr_pool_t *p)
     ap_assert(hPipeRead && hPipeWrite);
 
     stderr_ready = CreateEvent(NULL, FALSE, FALSE, NULL);
-    stderr_thread = CreateThread(NULL, 0, service_stderr_thread,
-                                 (LPVOID) hPipeRead, 0, &threadid);
+    stderr_thread = CreateThread(NULL, 65536, service_stderr_thread,
+                                 (LPVOID)hPipeRead, stack_res_flag, &threadid);
     ap_assert(stderr_ready && stderr_thread);
 
     WaitForSingleObject(stderr_ready, INFINITE);
 
-    if ((apr_file_open_stderr(&stderr_file, p) 
+    if ((apr_file_open_stderr(&stderr_file, p)
              == APR_SUCCESS)
      && (apr_os_file_put(&eventlog_file, &hPipeWrite, APR_WRITE, p)
              == APR_SUCCESS))
